@@ -3,7 +3,7 @@
  *                  (fill to fit parent's size) and using natural
  *                  size in other direction
  * 
- * Copyright 2012-2019 Stephan Haller <nomad@froevel.de>
+ * Copyright 2012-2020 Stephan Haller <nomad@froevel.de>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -436,6 +436,9 @@ static void _xfdashboard_fill_box_layout_allocate(ClutterLayoutManager *inLayout
 	clutter_actor_iter_init(&iter, CLUTTER_ACTOR(inContainer));
 	while(clutter_actor_iter_next(&iter, &child))
 	{
+		gboolean						fixedPosition;
+		gfloat							fixedX, fixedY;
+
 		/* Only set sizes on visible children */
 		if(!clutter_actor_is_visible(child)) continue;
 
@@ -470,9 +473,24 @@ static void _xfdashboard_fill_box_layout_allocate(ClutterLayoutManager *inLayout
 					}
 			}
 
-		/* Set new allocation of child */
-		childAllocation.x1=ceil(x);
-		childAllocation.y1=ceil(y);
+		/* Set new allocation of child but respect fixed position of actor */
+		g_object_get(child,
+						"fixed-position-set", &fixedPosition,
+						"fixed-x", &fixedX,
+						"fixed-y", &fixedY,
+						NULL);
+
+		if(fixedPosition)
+		{
+			childAllocation.x1=ceil(fixedX);
+			childAllocation.y1=ceil(fixedY);
+		}
+			else
+			{
+				childAllocation.x1=ceil(x);
+				childAllocation.y1=ceil(y);
+			}
+
 		childAllocation.x2=ceil(childAllocation.x1+w);
 		childAllocation.y2=ceil(childAllocation.y1+h);
 		clutter_actor_allocate(child, &childAllocation, inFlags);
